@@ -61,9 +61,21 @@ QString Clipboard::saveImage(QString localPath) const
 
 void Clipboard::saveText(QString message)
 {
-    QRegularExpression re("<[^>]*>");
+    static QRegularExpression re(QStringLiteral("<[^>]*>"));
     auto *mineData = new QMimeData; // ownership is transferred to clipboard
     mineData->setHtml(message);
-    mineData->setText(message.replace(re, ""));
+    mineData->setText(message.replace(re, QString()));
     m_clipboard->setMimeData(mineData);
 }
+
+void Clipboard::setImage(const QUrl &url)
+{
+    if (url.isLocalFile()) {
+        QImage img(url.path());
+        auto *mimeData = new QMimeData;
+        mimeData->setImageData(img);
+        if (!img.isNull()) {
+            m_clipboard->setMimeData(mimeData);
+        }
+    }
+}
