@@ -12,8 +12,8 @@
 
 #include <qcoro/task.h>
 
-class PollHandler;
-class NeoChatUser;
+#include "neochatuser.h"
+#include "pollhandler.h"
 
 class PushNotificationState : public QObject
 {
@@ -52,6 +52,19 @@ class NeoChatRoom : public Quotient::Room
     Q_PROPERTY(QString joinRule READ joinRule WRITE setJoinRule NOTIFY joinRuleChanged)
     Q_PROPERTY(QString historyVisibility READ historyVisibility WRITE setHistoryVisibility NOTIFY historyVisibilityChanged)
 
+    /**
+     * @brief Set the default URL preview state for room members.
+     *
+     * Assumed false if the org.matrix.room.preview_urls state message has never been
+     * set. Can only be set if the calling user has a high enough power level.
+     */
+    Q_PROPERTY(bool defaultUrlPreviewState READ defaultUrlPreviewState WRITE setDefaultUrlPreviewState NOTIFY defaultUrlPreviewStateChanged)
+
+    /**
+     * @brief Enable URL previews for the local user.
+     */
+    Q_PROPERTY(bool urlPreviewEnabled READ urlPreviewEnabled WRITE setUrlPreviewEnabled NOTIFY urlPreviewEnabledChanged)
+
     // Properties for the various permission levels for the room
     Q_PROPERTY(int defaultUserPowerLevel READ defaultUserPowerLevel WRITE setDefaultUserPowerLevel NOTIFY defaultUserPowerLevelChanged)
     Q_PROPERTY(int invitePowerLevel READ invitePowerLevel WRITE setInvitePowerLevel NOTIFY invitePowerLevelChanged)
@@ -79,6 +92,11 @@ class NeoChatRoom : public Quotient::Room
 
     // Due to problems with QTextDocument, unlike the other properties here, chatBoxText is *not* used to store the text when switching rooms
     Q_PROPERTY(QString chatBoxText READ chatBoxText WRITE setChatBoxText NOTIFY chatBoxTextChanged)
+
+    /**
+     * @brief The text for any message currently being edited in the room.
+     */
+    Q_PROPERTY(QString editText READ editText WRITE setEditText NOTIFY editTextChanged)
     Q_PROPERTY(QString chatBoxReplyId READ chatBoxReplyId WRITE setChatBoxReplyId NOTIFY chatBoxReplyIdChanged)
     Q_PROPERTY(QString chatBoxEditId READ chatBoxEditId WRITE setChatBoxEditId NOTIFY chatBoxEditIdChanged)
     Q_PROPERTY(NeoChatUser *chatBoxReplyUser READ chatBoxReplyUser NOTIFY chatBoxReplyIdChanged)
@@ -94,6 +112,7 @@ class NeoChatRoom : public Quotient::Room
      * Only returns main integer room versions (i.e. no msc room versions).
      */
     Q_PROPERTY(int maxRoomVersion READ maxRoomVersion NOTIFY maxRoomVersionChanged)
+    Q_PROPERTY(NeoChatUser *directChatRemoteUser READ directChatRemoteUser CONSTANT)
 
 public:
     enum MessageType {
@@ -112,13 +131,13 @@ public:
     /// This function respect the showLeaveJoinEvent setting and discard
     /// other not interesting events. This function can return an empty pointer
     /// when the room is empty of RoomMessageEvent.
-    [[nodiscard]] const Quotient::RoomEvent *lastEvent(bool ignoreStateEvent = false) const;
+    [[nodiscard]] const Quotient::RoomEvent *lastEvent() const;
 
     /// Convenient way to get the last event but in a string format.
     ///
     /// \see lastEvent
     /// \see lastEventIsSpoiler
-    [[nodiscard]] QString lastEventToString() const;
+    [[nodiscard]] QString lastEventToString(Qt::TextFormat format = Qt::PlainText, bool stripNewlines = false) const;
 
     /// Convenient way to check if the last event looks like it has spoilers.
     ///
@@ -131,12 +150,6 @@ public:
     /// \see lastEvent
     [[nodiscard]] QDateTime lastActiveTime();
 
-    /// Get subtitle text for room
-    ///
-    /// Fetches last event and removes markdown formatting
-    /// \see lastEventToString
-    [[nodiscard]] QString subtitleText();
-
     [[nodiscard]] bool isSpace();
 
     bool isEventHighlighted(const Quotient::RoomEvent *e) const;
@@ -147,6 +160,20 @@ public:
     [[nodiscard]] QString historyVisibility() const;
     void setHistoryVisibility(const QString &historyVisibilityRule);
 
+    [[nodiscard]] bool defaultUrlPreviewState() const;
+    void setDefaultUrlPreviewState(const bool &defaultUrlPreviewState);
+
+    [[nodiscard]] bool urlPreviewEnabled() const;
+    void setUrlPreviewEnabled(const bool &urlPreviewEnabled);
+
+    /**
+     * @brief Get the power level for the given user ID in the room.
+     *
+     * Returns the default value for a user in the room if they have no escalated
+     * privileges or if they are not a member so membership should be known before using.
+     */
+    Q_INVOKABLE [[nodiscard]] int getUserPowerLevel(const QString &userId) const;
+
     Q_INVOKABLE void setUserPowerLevel(const QString &userID, const int &powerLevel);
 
     [[nodiscard]] int powerLevel(const QString &eventName, const bool &isStateEvent = false) const;
@@ -248,7 +275,8 @@ public:
 
     [[nodiscard]] QString avatarMediaId() const;
 
-    [[nodiscard]] QString eventToString(const Quotient::RoomEvent &evt, Qt::TextFormat format = Qt::PlainText, bool removeReply = true) const;
+    [[nodiscard]] QString eventToString(const Quotient::RoomEvent &evt, Qt::TextFormat format = Qt::PlainText, bool stripNewlines = false) const;
+    [[nodiscard]] QString eventToGenericString(const Quotient::RoomEvent &evt) const;
 
     Q_INVOKABLE [[nodiscard]] bool containsUser(const QString &userID) const;
     Q_INVOKABLE [[nodiscard]] bool isUserBanned(const QString &user) const;
@@ -270,6 +298,9 @@ public:
     QString chatBoxText() const;
     void setChatBoxText(const QString &text);
 
+    QString editText() const;
+    void setEditText(const QString &text);
+
     QString chatBoxReplyId() const;
     void setChatBoxReplyId(const QString &replyId);
 
@@ -287,6 +318,11 @@ public:
 
     QVector<Mention> *mentions();
 
+    /**
+     * @brief Vector of mentions in the current edit text.
+     */
+    QVector<Mention> *editMentions();
+
     QString savedText() const;
     void setSavedText(const QString &savedText);
 
@@ -316,6 +352,7 @@ public:
 #endif
 
     int maxRoomVersion() const;
+    NeoChatUser *directChatRemoteUser() const;
 
 private:
     QSet<const Quotient::RoomEvent *> highlights;
@@ -336,10 +373,12 @@ private:
     QCoro::Task<void> doUploadFile(QUrl url, QString body = QString());
 
     QString m_chatBoxText;
+    QString m_editText;
     QString m_chatBoxReplyId;
     QString m_chatBoxEditId;
     QString m_chatBoxAttachmentPath;
     QVector<Mention> m_mentions;
+    QVector<Mention> m_editMentions;
     QString m_savedText;
 #ifdef QUOTIENT_07
     QCache<QString, PollHandler> m_polls;
@@ -362,12 +401,15 @@ Q_SIGNALS:
     void pushNotificationStateChanged(PushNotificationState::State state);
     void showMessage(MessageType messageType, const QString &message);
     void chatBoxTextChanged();
+    void editTextChanged();
     void chatBoxReplyIdChanged();
     void chatBoxEditIdChanged();
     void chatBoxAttachmentPathChanged();
     void canEncryptRoomChanged();
     void joinRuleChanged();
     void historyVisibilityChanged();
+    void defaultUrlPreviewStateChanged();
+    void urlPreviewEnabledChanged();
     void maxRoomVersionChanged();
     void defaultUserPowerLevelChanged();
     void invitePowerLevelChanged();
